/***************************************************************************
                          wvoxbase.cpp  -  description
                             -------------------
    begin                : 2004ǯ  5  4  17:14:12 JST
    copyright            : (C) 2004 by Tomoaki Murakami
    email                : solarisphere@yahoo.co.jp
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/////////////////////////////////////////////////////////////////////////////

#include <qlabel.h>
#include <qlayout.h>
#include <qscrollview.h>
#include <qsplitter.h>
#include <klocale.h>
#include <kmessagebox.h>
#include "wvoxbase.h"
#include "wnamedialog.h"
#include "cvoxstormglobal.h"

/////////////////////////////////////////////////////////////////////////////

WVoxBase::WVoxBase(QWidget *WidgetParent, const char *cName)
  : QVBox(WidgetParent, cName)
{
  setupWidgets();
}

/////////////////////////////////////////////////////////////////////////////

WVoxBase::WVoxBase(
      const QDomNode& nodBase,
      QWidget*        WidgetParent,
      const char*     cName)
  : QVBox(WidgetParent, cName)
{
  setupWidgets();
  addTracks(nodBase);
}

/////////////////////////////////////////////////////////////////////////////

WVoxBase::~WVoxBase()
{
  if (vVoxTracks.empty() ) {
    return;
  }
  int i, nTrack;

  nTrack = vVoxTracks.size();
  for ( i = 0; i < nTrack; i++ ) {
    delete vVoxTracks[i];
    vVoxTracks[i] = NULL;
  }

  vVoxTracks.clear();
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::addTrack(const QString& strDeviceIn, const QString& strDeviceOut)
{
  WNameDialog *NameDialog = new WNameDialog(this);

  NameDialog->setValidChars(CVoxStormGlobal::ValidCharsTrack);

  if ( NameDialog->exec() == QDialog::Accepted ) {
    QString strTrackName = NameDialog->getNameFromDialog();
    
    if ( queryExistingTrackName(strTrackName)) { // already exsists
      QString strMessage = "The name \"";
      strMessage += strTrackName;
      strMessage += "\" is already existing.\nFailed in adding track.";
      KMessageBox::sorry(this, strMessage);

    } else {
      addTrack(strTrackName, strDeviceIn, strDeviceOut);
    }
  }
  
  delete NameDialog;
}

/////////////////////////////////////////////////////////////////////////////

void 
WVoxBase::addTrack(
            const QString& strTrackName, 
            const QString& strDeviceIn,
            const QString& strDeviceOut)
{
  WVoxTrack *VoxTrackNew = new WVoxTrack(
                                  strTrackName,
                                  strDeviceIn,
                                  strDeviceOut,
                                  VBoxTracks);

  SignalMapperPlayback->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(playbackDeviceChanged(const QString&)),
          SignalMapperPlayback, SLOT(map()));

  SignalMapperRecording->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(recordingDeviceChanged(const QString&)),
          SignalMapperRecording, SLOT(map()));

  SignalMapperRemove->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(removed()), SignalMapperRemove, SLOT(map()));

  SignalMapperFileName->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(fileNameChanged(const QString&)),
          SignalMapperFileName, SLOT(map()));

  SignalMapperLevel->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(levelChanged(int)), SignalMapperLevel, SLOT(map()));

  SignalMapperPan->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(panChanged(int)), SignalMapperPan, SLOT(map()));

  SignalMapperMute->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(muteToggled(bool)), SignalMapperMute, SLOT(map()));

  SignalMapperRec->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(recToggled(bool)), SignalMapperRec, SLOT(map()));

  SignalMapperStartPosition->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(startPositionChanged(const QTime&)),
          SignalMapperStartPosition, SLOT(map()));

  SignalMapperOffset->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(offsetChanged(const QTime&)),
          SignalMapperOffset, SLOT(map()));

  SignalMapperLength->setMapping(VoxTrackNew, strTrackName);
  connect(VoxTrackNew, SIGNAL(lengthChanged(const QTime&)), 
          SignalMapperLength, SLOT(map()));

  vVoxTracks.push_back(VoxTrackNew);
  VoxTrackNew->show();
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::addTracks(const QDomNode& nodTracks)
{
  if ( nodTracks.isNull() ) {
    return;
  }

  QDomNodeList nolTracks = nodTracks.childNodes();
  int i;
  int nCount = nolTracks.count();
  for ( i = 0; i < nCount; i++ ) {
    WVoxTrack *VoxTrackNew = new WVoxTrack(nolTracks.item(i), VBoxTracks);
    QString strTrackName = VoxTrackNew->name();

    SignalMapperPlayback->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(playbackDeviceChanged(const QString&)),
            SignalMapperPlayback, SLOT(map()));

    SignalMapperRecording->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(recordingDeviceChanged(const QString&)),
            SignalMapperRecording, SLOT(map()));

    SignalMapperRemove->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(removed()), SignalMapperRemove, SLOT(map()));

    SignalMapperFileName->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(fileNameChanged(const QString&)), 
            SignalMapperFileName, SLOT(map()));

    SignalMapperLevel->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(levelChanged(int)), SignalMapperLevel, SLOT(map()));

    SignalMapperPan->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(panChanged(int)), SignalMapperPan, SLOT(map()));

    SignalMapperMute->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(muteToggled(bool)), SignalMapperMute, SLOT(map()));

    SignalMapperRec->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(recToggled(bool)), SignalMapperRec, SLOT(map()));

    SignalMapperStartPosition->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(startPositionChanged(const QTime&)),
            SignalMapperStartPosition, SLOT(map()));

    SignalMapperOffset->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(offsetChanged(const QTime&)),
            SignalMapperOffset, SLOT(map()));

    SignalMapperLength->setMapping(VoxTrackNew, strTrackName);
    connect(VoxTrackNew, SIGNAL(lengthChanged(const QTime&)),
            SignalMapperLength, SLOT(map()));

    vVoxTracks.push_back(VoxTrackNew);
    VoxTrackNew->show();
  }
}

/////////////////////////////////////////////////////////////////////////////

void 
WVoxBase::clearTracks()
{
  if ( vVoxTracks.empty() ) {
    return;
  }

  int i, nTrack;

  nTrack = vVoxTracks.size();
  for ( i = 0; i < nTrack; i++ ) {
    SignalMapperPlayback->removeMappings(vVoxTracks[i]);
    SignalMapperRecording->removeMappings(vVoxTracks[i]);
    SignalMapperRemove->removeMappings(vVoxTracks[i]);
    SignalMapperFileName->removeMappings(vVoxTracks[i]);
    SignalMapperLevel->removeMappings(vVoxTracks[i]);
    SignalMapperPan->removeMappings(vVoxTracks[i]);
    SignalMapperMute->removeMappings(vVoxTracks[i]);
    SignalMapperRec->removeMappings(vVoxTracks[i]);
    SignalMapperStartPosition->removeMappings(vVoxTracks[i]);
    SignalMapperOffset->removeMappings(vVoxTracks[i]);
    SignalMapperLength->removeMappings(vVoxTracks[i]);

    delete vVoxTracks[i];
    vVoxTracks[i] = NULL;
  }
  if ( nTrack != 0 ) {
    vVoxTracks.clear();
  }
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::removeTrack()
{
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::removeTrack(const QString& strTrackName)
{
  int nID;

  if ( (nID = getIndexByName(strTrackName)) == -1 ) {  // not found
    return;
  }

  QValueVector<WVoxTrack*> vVoxTracksTmp;
  int i, nTrack;
  
  nTrack = vVoxTracks.size();
  for ( i = 0; i < nTrack; i++ ) {
    if ( i == nID ) {  // subject track
      SignalMapperPlayback->removeMappings(vVoxTracks[i]);
      SignalMapperRecording->removeMappings(vVoxTracks[i]);
      SignalMapperRemove->removeMappings(vVoxTracks[i]);
      SignalMapperFileName->removeMappings(vVoxTracks[i]);
      SignalMapperLevel->removeMappings(vVoxTracks[i]);
      SignalMapperPan->removeMappings(vVoxTracks[i]);
      SignalMapperMute->removeMappings(vVoxTracks[i]);
      SignalMapperRec->removeMappings(vVoxTracks[i]);
      SignalMapperStartPosition->removeMappings(vVoxTracks[i]);
      SignalMapperOffset->removeMappings(vVoxTracks[i]);
      SignalMapperLength->removeMappings(vVoxTracks[i]);
      delete vVoxTracks[i];
    } else {
      vVoxTracksTmp.push_back(vVoxTracks[i]);
    }
  }

  vVoxTracks.clear();
  nTrack--;
  for ( i = 0; i < nTrack; i++ ) {
    vVoxTracks.push_back(vVoxTracksTmp[i]);
  }
  vVoxTracksTmp.clear();
}

/////////////////////////////////////////////////////////////////////////////

int
WVoxBase::getTrackCount() const
{
  return vVoxTracks.size();
}

/////////////////////////////////////////////////////////////////////////////

CTrackInf 
WVoxBase::getTrackInf(int nID) const
{
  CTrackInf TrackInfReturn;
  if ( (0 > nID) || ((int)vVoxTracks.size() <= nID) ) {
    return TrackInfReturn;
  }

  return vVoxTracks[nID]->stateInTrackInf();
}

/////////////////////////////////////////////////////////////////////////////

CTrackInf
WVoxBase::getTrackInf(const QString& strTrackName) const
{
  return getTrackInf(getIndexByName(strTrackName));
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::setupWidgets()
{
  setFrameStyle(QFrame::Box | QFrame::Sunken);
  setMinimumWidth(512);
  setMinimumHeight(400);

  SignalMapperPlayback = new QSignalMapper(this);
  connect(SignalMapperPlayback, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangePlaybackDevice(const QString&)));

  SignalMapperRecording = new QSignalMapper(this);
  connect(SignalMapperRecording, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeRecordingDevice(const QString&)));

  SignalMapperRemove = new QSignalMapper(this);
  connect(SignalMapperRemove, SIGNAL(mapped(const QString&)),
          this, SLOT(slotRemove(const QString&)));

  SignalMapperFileName = new QSignalMapper(this);
  connect(SignalMapperFileName, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeFileName(const QString&)));

  SignalMapperLevel = new QSignalMapper(this);
  connect(SignalMapperLevel, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeLevel(const QString&)));

  SignalMapperPan = new QSignalMapper(this);
  connect(SignalMapperPan, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangePan(const QString&)));

  SignalMapperMute = new QSignalMapper(this);
  connect(SignalMapperMute, SIGNAL(mapped(const QString&)),
          this, SLOT(slotToggleMute(const QString&)));

  SignalMapperRec = new QSignalMapper(this);
  connect(SignalMapperRec, SIGNAL(mapped(const QString&)),
          this, SLOT(slotToggleRec(const QString&)));

  SignalMapperStartPosition = new QSignalMapper(this);
  connect(SignalMapperStartPosition, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeStartPosition(const QString&)));

  SignalMapperOffset = new QSignalMapper(this);
  connect(SignalMapperOffset, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeOffset(const QString&)));

  SignalMapperLength = new QSignalMapper(this);
  connect(SignalMapperLength, SIGNAL(mapped(const QString&)),
          this, SLOT(slotChangeLength(const QString&)));

  QVBox       *VBoxBase = new QVBox(this);
  QWidget     *WidgetMisc = new QWidget(VBoxBase);
  QHBoxLayout *HBoxLayout = new QHBoxLayout(WidgetMisc);
  TimeEditPosition = new QTimeEdit(WidgetMisc);
  TimeEditPosition->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
  connect(TimeEditPosition, SIGNAL(valueChanged(const QTime&)),
          this, SIGNAL(positionChanged(const QTime&)));
  HBoxLayout->addWidget(TimeEditPosition);
  TimeEditPosition->setDisplay(
      QTimeEdit::Minutes | QTimeEdit::Seconds);
  HBoxLayout->addStretch();

  QSplitter   *Splitter = new QSplitter(Vertical, VBoxBase);
  QScrollView *ScrollView = new QScrollView(Splitter);

  VBoxTracks = new QVBox(Splitter);
  VBoxTracks->setMinimumHeight(300);
  ScrollView->addChild(VBoxTracks);

  Tab = new QTabWidget(Splitter);
  Tab->setTabPosition(QTabWidget::Bottom);
  //  TextEditCmdLine = new KTextEdit(Tab);
  //  Tab->addTab(TextEditCmdLine, "Commandline");
  TextEditEcasound = new KTextEdit(Tab);
  TextEditEcasound->setReadOnly(true);
  Tab->addTab(TextEditEcasound, "Ecasound console");
}

/////////////////////////////////////////////////////////////////////////////

int 
WVoxBase::getIndexByName(const QString& strTrackName) const
{
  int i, nTrack;

  nTrack = vVoxTracks.size();  
  for ( i = 0; i < nTrack; i++ ) {
    if ( strTrackName == vVoxTracks[i]->name() ) {
      return i;
    }
  }

  return -1;
}

/////////////////////////////////////////////////////////////////////////////

bool
WVoxBase::queryExistingTrackName(const QString& strName) const
{
  int i, nTrack;
  
  nTrack = vVoxTracks.size();
  for ( i = 0; i < nTrack; i++ ) {
    if ( vVoxTracks[i]->name() == strName ) {
      return true;
    }
  }

  return false;
}

/////////////////////////////////////////////////////////////////////////////

void 
WVoxBase::setEcasoundCmd(const QString& strCmd)
{
  TextEditCmdLine->clear();
  TextEditCmdLine->setText(strCmd);
}

/////////////////////////////////////////////////////////////////////////////

QString
WVoxBase::ecasoundCmd() const
{
  return TextEditCmdLine->text();
}

/////////////////////////////////////////////////////////////////////////////
QDomNode
WVoxBase::stateInDOM(QDomDocument* pDoc)
{
  QDomElement eleTracks = pDoc->createElement("Tracks");
  QValueVector<WVoxTrack*>::iterator it = vVoxTracks.begin();
  while ( it != vVoxTracks.end() ) {
    eleTracks.appendChild((*it)->stateInDOM(pDoc));
    it++;
  }

  return eleTracks;
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotWriteConsoleMessage(const QString& strMessage)
{
  TextEditEcasound->append(strMessage);
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeMessageTab(TabType type)
{
  if ( type == WVoxBase::CommandLine ) {
    Tab->setCurrentPage(0);
  } else if ( type == WVoxBase::Console ) {
    Tab->setCurrentPage(0);
  }
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotSetEnabledWidgets(State state)
{
  int i, nTracks;

  nTracks = vVoxTracks.size();
  if ( state == Play ) {
    for ( i = 0; i < nTracks; i++ ) {
      vVoxTracks[i]->slotSetEnabledWidgets(
                       WVoxTrack::Record |
                       WVoxTrack::File |
                       WVoxTrack::Remove |
                       WVoxTrack::Device |
                       WVoxTrack::StartPosition |
                       WVoxTrack::Offset |
                       WVoxTrack::Length,
                       false);
    }
  } else if ( state == Pause ) {
    for ( i = 0; i < nTracks; i++ ) {
      vVoxTracks[i]->slotSetEnabledWidgets(WVoxTrack::Record |
                       WVoxTrack::File |
                       WVoxTrack::Remove |
                       WVoxTrack::Device,
                       true);
      vVoxTracks[i]->slotSetEnabledWidgets(WVoxTrack::Record |
                       WVoxTrack::StartPosition |
                       WVoxTrack::Offset |
                       WVoxTrack::Length,
                       false);
    }
  } else if ( state == Stop ) {
    for ( i = 0; i < nTracks; i++ ) {
      vVoxTracks[i]->slotSetEnabledWidgets(
                       WVoxTrack::Record |
                       WVoxTrack::File |
                       WVoxTrack::Remove |
                       WVoxTrack::Device |
                       WVoxTrack::StartPosition |
                       WVoxTrack::Offset |
                       WVoxTrack::Length,
                       true);
    }
  }    
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotNotifySucceedInRecording()
{
  QValueVector<WVoxTrack*>::iterator it = vVoxTracks.begin();

  while ( it != vVoxTracks.end() ) {
    if ( (*it)->isSetRec() ) {
      (*it)->setStartPosition(QTime(0,0));
      (*it)->setOffset(QTime(0,0));
      (*it)->setLength(QTime(0,0));
    }
    it++;
  }  
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotAddTrack()
{
  addTrack("/dev/dsp", "/dev/dsp");
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangePosition(const QTime& Time)
{
  TimeEditPosition->setTime(Time);
}

/////////////////////////////////////////////////////////////////////////////

void 
WVoxBase::slotChangePlaybackDevice(const QString& strTrackName)
{
  emit playbackDeviceChanged(strTrackName,
                       vVoxTracks[getIndexByName(strTrackName)]->playbackDevice());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeRecordingDevice(const QString& strTrackName)
{
  emit recordingDeviceChanged(strTrackName,
                       vVoxTracks[getIndexByName(strTrackName)]->recordingDevice());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotRemove(const QString& strTrackName)
{
  removeTrack(strTrackName);
  emit trackRemoved(strTrackName);
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeFileName(const QString& strTrackName)
{
  emit fileNameChanged(strTrackName,
                       vVoxTracks[getIndexByName(strTrackName)]->fileName());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeLevel(const QString& strTrackName)
{
  emit levelChanged(strTrackName,
		    vVoxTracks[getIndexByName(strTrackName)]->level());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangePan(const QString& strTrackName)
{
  emit panChanged(strTrackName,
		  vVoxTracks[getIndexByName(strTrackName)]->pan());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotToggleMute(const QString& strTrackName)
{
  emit muteToggled(strTrackName,
		   vVoxTracks[getIndexByName(strTrackName)]->isMuted());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotToggleRec(const QString& strTrackName)
{
  emit recToggled(strTrackName,
		  vVoxTracks[getIndexByName(strTrackName)]->isSetRec());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeStartPosition(const QString& strTrackName)
{
  emit startPositionChanged(strTrackName,
		  vVoxTracks[getIndexByName(strTrackName)]->startPosition());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeOffset(const QString& strTrackName)
{
  emit offsetChanged(strTrackName,
		  vVoxTracks[getIndexByName(strTrackName)]->offset());
}

/////////////////////////////////////////////////////////////////////////////

void
WVoxBase::slotChangeLength(const QString& strTrackName)
{
  emit lengthChanged(strTrackName,
		  vVoxTracks[getIndexByName(strTrackName)]->length());
}

/////////////////////////////////////////////////////////////////////////////

#include "wvoxbase.moc"
